#if defined(WIN32)
#ifdef OPENSSLWRAPPER_EXPORTS
#define OPENSSLWRAPPER_API __declspec(dllexport)
#else
#define OPENSSLWRAPPER_API __declspec(dllimport)
#endif
#else
#define OPENSSLWRAPPER_API
#endif //WIN32

// revival encrypto

class OPENSSLWRAPPER_API OpenSSL_AES
{
public:
	OpenSSL_AES();
	~OpenSSL_AES();

	int GenerateNewCryptKey();

	const BYTE * GetCryptKey();
	bool SetCryptKey(const BYTE * pKey);

	const BYTE * GetInitialVector();
	bool SetInitialVector(const BYTE * pKey);

	int Encrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer);
	int Decrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer);
	
	int EncryptedSizeGuess(int nSourceDataSize);

private:
	void* m_delegator;
};

class OPENSSLWRAPPER_API OpenSSL_RSA
{
public:
	OpenSSL_RSA();
	~OpenSSL_RSA();

public:
	static int sRSA_EncryptByPublicKey(const BYTE * pRSA_PublicKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);
	static int sRSA_DecryptByPublicKey(const BYTE * pRSA_PublicKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);
	static int sRSA_EncryptByPrivateKey(const BYTE * pRSA_PrivateKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);
	static int sRSA_DecryptByPrivateKey(const BYTE * pRSA_PrivateKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);

public:
	bool Create_InternalKey(int seed);
	bool Initialize_By_ExternalKey(const BYTE * pRSA_PublicKey, int nPubKeyLen, const BYTE * pRSA_PrivateKey, int nPriKeyLen);

	int GetPublicKey(BYTE ** ppOutBuffer, void* (*mem_allocator)(size_t));
	int GetPrivateKey(BYTE ** ppOutBuffer, void* (*mem_allocator)(size_t));
	
	int Public_Encrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);
	int Public_Decrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);

	int Private_Encrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);
	int Private_Decrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen);

private:
	void* m_delegator;
};

class OPENSSLWRAPPER_API OpenSSL_Crypt
{
public:
	OpenSSL_Crypt();
	~OpenSSL_Crypt();

	void Init(int nEncType, const BYTE * pAesCryptKey = NULL);
	int GetEncType();

	int GetAesKeySize();
	const BYTE * GetAesKey();

	int AesEncrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer);
	int AesDecrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer);

	int AesEncryptSizeGuess(int nSrcLen);

private:
	void* m_delegator;
};
